import { EOrientation } from "../types"
import { LANDSCAPE_WIDTH, PORTRAIT_WIDTH } from './Page'

type TableElement = 'table-header' | 'table-body' | 'table-row' | 'table-cell'

export class TableTransfrom {
  private orientation: EOrientation

  constructor(orientation: EOrientation) {
    this.orientation = orientation
  }

  // Трансформирует таблицу из table тега, в таблицу из div
  tableTransform(elem: HTMLDivElement, scale: number = 1) {

    // Создаем строку для шаблона
    const divTemplateRow = document.createElement('div')

    // Создаем таблицу со всеми атрибутами
    const divTable = elem.querySelector('.table')!.cloneNode() as HTMLDivElement
    divTemplateRow.appendChild(divTable)

    // Вставляем в наш скрытый дом, для вычисления ширины в будущем
    const iframe = document.querySelector('.iframe-hidden-dom') as HTMLIFrameElement
    iframe.contentDocument?.body.querySelector('.hidden-dom .template-constructor')?.appendChild(divTemplateRow)

    // Транформируем нашу таблицу из табличного вида в div вид
    const divHeader = this.transformHeader(elem.querySelector('thead') as HTMLTableSectionElement)
    const divBody = this.transformBody(elem.querySelector('tbody') as HTMLTableSectionElement)

    divTable.appendChild(divHeader)
    divTable.appendChild(divBody)

    const maxWidth = this.tableInspect(divTable)

    this.tableShrink(divTable, maxWidth)

    return divTemplateRow
  }

  // Создает  элемент для таблицы хедер,боди,строку или ячейку
  createTableElement(type: TableElement) {
    const divElement = document.createElement('div')
    divElement.classList.add(type)
    return divElement
  }

  transformRow(row: HTMLTableRowElement) {
    const tableCells = row.cells;
    const divRow = this.createTableElement('table-row');
    [...tableCells].forEach((tableCell) => {

      const divCell = this.createTableElement('table-cell')
      tableCell.childNodes.forEach(cellChildNode => divCell.appendChild(cellChildNode.cloneNode(true)))

      divRow.appendChild(divCell)
    })

    return divRow
  }

  transformHeader(tableHeader: HTMLTableSectionElement) {

    const tableRow = tableHeader.querySelector('tr') as HTMLTableRowElement

    const divHeader = this.createTableElement('table-header')
    const divRow = this.transformRow(tableRow)

    divHeader.appendChild(divRow)

    return divHeader
  }

  transformBody(tableBody: HTMLTableSectionElement) {
    const tableRows = [...tableBody.querySelectorAll('tr')];

    const divBody = this.createTableElement('table-body')
    const divRows = tableRows.map((e) => this.transformRow(e))

    divRows.forEach((e) => divBody.appendChild(e))

    return divBody
  }

  getTableElement(tableElement: HTMLDivElement, selector: string) {
    return [...tableElement.querySelectorAll(`${selector}`) ?? []]
  }

  // Выдает максимальну ширину ячеек по каждому столбцу
  tableInspect(divTable: HTMLDivElement): number[] {

    const divRows = this.getTableElement(divTable, '.table-row') as HTMLDivElement[]

    const countColl = this.getTableElement(divRows[0], '.table-cell').length

    const maxWidth: number[] = Array(countColl).fill(0)

    divRows.forEach(row => {
      const divCells = this.getTableElement(row, '.table-cell') as HTMLDivElement[]

      divCells.forEach((cell, index) => {
        // подписант встает в одну строку в таблице без scale, поэтому задаем отдельную ширину
        // иначе портит сумму ширин ячеек в методе tableShrink
        if (cell.querySelector('.signer')) {
          maxWidth[index] = Math.max(maxWidth[index] ?? 0, 200)
          return
        }
        maxWidth[index] = Math.max(maxWidth[index] ?? 0, cell.offsetWidth)
      })
    })

    return maxWidth
  }

  tableShrink(divTable: HTMLDivElement, maxWidth: number[]) {
    const maxWidthPage = this.orientation == EOrientation.portrait ? PORTRAIT_WIDTH : LANDSCAPE_WIDTH
    const sumWidthColls = maxWidth.reduce((sum, width) => sum + width, 0)

    const scale = maxWidthPage / sumWidthColls

    const divRows = this.getTableElement(divTable, '.table-row') as HTMLDivElement[]

    divRows.forEach(row => {
      const divCells = this.getTableElement(row, '.table-cell') as HTMLDivElement[]

      // Если одна из ячеек в таблице после скейла имеет ширину меньше 10px
      // То мы берем разницу между 10px и ее шириной
      // После мы забираем эту ширину у следующей ячейки, которую дали прошлой
      let distributeWidth = 0

      divCells.forEach((cell, index) => {

        if (scale < 1) {
          cell.style.lineHeight = `${20.72 * (scale + 0.2)}pt`
          cell.style.fontSize = `${14 * (scale + 0.2)}pt`

          const img = cell.querySelector('img')
          if (img)
            img.style.width = `${100 * (scale + 0.2)}px`
        }

        if (distributeWidth && maxWidth[index] * scale - 5 - distributeWidth > 10) {
          cell.style.width = `${maxWidth[index] * scale - 5 - distributeWidth}px`
          distributeWidth = 0
          return
        }


        if (maxWidth[index] * scale - 5 < 10 || (distributeWidth && maxWidth[index] * scale - 5 - distributeWidth < 10)) {
          cell.style.width = `10px`
          distributeWidth += 10 - (maxWidth[index] * scale - 5)
          return
        }

        cell.style.width = `${maxWidth[index] * scale - 5}px`
      })
    })
  }

}